import fs from 'fs';
import path from 'path';
import ts from 'typescript';
import traverseAndUpdateFileWithJSExtension from './traverse-and-update.js';
import { ExtensionsUtil, separator } from './const.js';
import Log from './log.js';
const getAllJSAndDTSFiles = (directory) => {
    return fs.readdirSync(directory).flatMap((file) => {
        const filePath = path.posix.join(directory.split(path.sep).join(separator), file);
        if (fs.statSync(filePath).isDirectory()) {
            return getAllJSAndDTSFiles(filePath);
        }
        return !ExtensionsUtil.matchEither(filePath) ? [] : [filePath];
    });
};
const readCode = (file) => {
    return new Promise((resolve, reject) => {
        let fetchData = '';
        fs.createReadStream(file)
            .on('data', (data) => {
            fetchData = data.toString();
        })
            .on('end', () => {
            resolve(fetchData);
        })
            .on('error', reject);
    });
};
const getAllJSAndDTSCodes = (files) => {
    return files.map(async (file) => {
        const code = await readCode(file);
        return {
            code,
            parsed: ts.createSourceFile(file, code, ts.ScriptTarget.ESNext),
        };
    });
};
export default class File {
    constructor() { }
    static create = () => {
        return new this();
    };
    findMany = async (props) => {
        // user may import files from `common` into `src`
        const files = props.include
            .concat(props.dir)
            .flatMap(getAllJSAndDTSFiles);
        return (await Promise.all(getAllJSAndDTSCodes(files))).flatMap(traverseAndUpdateFileWithJSExtension(files));
    };
    writeMany = async (props) => {
        const repeat = props.withJSExtension.reduce((longestFileName, { file }) => {
            return longestFileName?.length <= file.length
                ? file
                : longestFileName;
        }, '').length;
        const log = !(props.withJSExtension.length && props.showChanges)
            ? undefined
            : Log.fromNumberOfFiles(props.withJSExtension.length);
        try {
            const errors = (await Promise.all(props.withJSExtension.map(({ code, file }) => {
                return new Promise((resolve) => {
                    return fs.writeFile(file, code, (error) => {
                        log?.increment({
                            repeat,
                            file,
                            succeed: !error,
                        });
                        resolve(!error
                            ? undefined
                            : {
                                file,
                                error,
                            });
                    });
                });
            }))).flatMap((element) => {
                return !element ? [] : [element];
            });
            log?.end({ errors });
            return {
                type: 'done',
            };
        }
        catch (error) {
            return {
                type: 'error',
                error,
            };
        }
    };
}
//# sourceMappingURL=read-write.js.map